#!/usr/bin/env python3
"""
   utils.database.from_api.py file for the c3 buttons.
"""


###
### DEPENDENCIES
###
import sys
import requests

### LOCAL
try:
    from .from_database import DataBaseUsed
except ImportError as e:
    print("Unable to import 'DataBaseUsed' from 'utils.database.from_database.py' file at 'utils.database.from_api.py' file" )
    sys.exit(1)


class DataBaseApi(DataBaseUsed):
    """
        TODO: Documentation
    """

    def __init__(self, configuration: dict) -> None:
        """
            TODO: Documentation
        """

        ### Init super
        super().__init__(name=__name__, configuration=configuration)

        self.__check_token__()

    def __check_token__(self) -> None:
        """
            TODO: Documentation
        """

        self.logger.debug("__check_token__")

        response = self.get(endpoint="auth/test/closed")
        if response is None:
            self.logger.error("__check_token__ - Token is NOT working...")
            print("\nERROR ---> Initial Token is not working.\n You should Generate a new one and set in the configuration...\n\n")
            sys.exit()

        else:
            if response.get('error'):
                print("\nERROR ---> Some error happend trying to get the initial token.\n ")
                sys.exit()

            else:
                self.logger.info("__check_token__ - Token is working...")

    def __get_token_headers__(self) -> dict:

        return {
            "Content-Type": "application/json; charset=utf-8",
            "Authorization": f"Bearer {self.configuration.database_api_token}"
        }

    def get(self, endpoint, params=None) -> (dict | None):

        self.logger.debug(f"get - endpoint: '{endpoint}' | params: '{params}'")

        url = f"{self.configuration.database_api_url}/{endpoint}"
        try:
            response = requests.get(
                url,
                headers=self.__get_token_headers__(),
                params=params
            )
            response.raise_for_status()
            return response.json()

        except requests.RequestException as e:
            self.logger.error(f"GET request failed with exception: {e}")
            return None

    def post(self, endpoint, json=None, data=None) -> (dict | None):

        self.logger.debug(f"post - endpoint: '{endpoint}' | json: '{json}' | data: '{data}'")

        url = f"{self.configuration.database_api_url}/{endpoint}"
        try:
            response = requests.post(
                url,
                json=json,
                data=data,
                headers=self.__get_token_headers__()
            )
            response.raise_for_status()
            return response.json()

        except requests.RequestException as e:
            self.logger.error(f"POST request failed with exception: {e}")
            return None

    def add_button_box_usage(self, box_name: str):

        self.logger.debug(f"add_button_box_usage - box_name: '{box_name}'")

        post_data = {
            "name": box_name
        }
        resp = self.post(endpoint="api/box/use", json=post_data)
        if resp.get('error'):
            self.logger.error(f"Error adding button usage with response: {resp}")
