#!/usr/bin/env python3
"""
   utils.database.from_local_file.py file for the c3 buttons.
"""

###
### DEPENDENCIES
###
import sys
import sqlite3
from datetime import datetime

### LOCAL
try:
    from .from_database import DataBaseUsed
except ImportError as e:
    print("Unable to import 'DataBaseUsed' from 'utils.database.from_database.py' file at 'utils.database.from_local_file.py' file" )
    sys.exit(1)


class DataBaseLocalSqLite(DataBaseUsed):
    """ TODO
    """

    def __init__(self, configuration: dict) -> None:

        ### Init super
        super().__init__(name=__name__, configuration=configuration)

        self.file_path = self.configuration.database_local_db_file_path

    ###
    ### CONNECTION
    ###
    def conn_create(self):
        self.conn = sqlite3.connect(self.file_path)
        self.cursor = self.conn.cursor()

    def conn_close(self):

        if self.conn:
            self.cursor.close()
            self.conn.close()
            self.cursor = None
            self.conn = None

    ###
    ### QUERY
    ###
    def query_fetch_one(self, query: str) -> None:

        # self.logger.debug(f"query_fetch_one - query: {query}")
        self.logger.debug("query_fetch_one")

        if self.conn is None:
            self.conn_create()

        try:
            resp = self.cursor.execute(query)
            return resp.fetchone()[0]

        except Exception as e:
            self.logger.error(f"ERROR executing query: {query} | with execption: {e}")

    def query_fetch_all(self, query: str) -> None:

        # self.logger.debug(f"query_fetch_all - query: {query}")
        self.logger.debug("query_fetch_all")

        if self.conn is None:
            self.conn_create()

        try:
            self.cursor.execute(query)
            resp = self.cursor.execute(query)

            return resp.fetchall()

        except Exception as e:
            self.logger.error(f"ERROR executing query: {query} | with execption: {e}")

        finally:
            self.conn_close()

    def query_exec(self, query: str) -> None:

        # self.logger.debug(f"query_exec - query: {query}")
        self.logger.debug("query_exec")

        if self.conn is None:
            self.conn_create()

        try:
            self.cursor.execute(query)

        except Exception as e:
            self.logger.error(f"ERROR executing query: {query} | with execption: {e}")

        finally:
            if self.conn:
                self.conn.commit()

    ###
    ### TABLES
    ###
    def table_select_all(self, table_name: str) -> (list | None):

        self.logger.debug(f"table_select_all - table_name: {table_name}")

        if self.conn is None:
            self.conn_create()

        query = "SELECT * FROM " + table_name + ";"

        return self.query_fetch_all(query=query)

    def table_get_button_box_id(self, box_name: str):

        self.logger.debug(f"table_get_button_box_id - box_name '{box_name}'")

        table_name = self.configuration.database_table_name_button_box

        query = """
        SELECT id FROM {table_name} WHERE {field_name}="{field_value}";
        """.format(
            table_name=table_name,
            field_name="name",
            field_value=box_name
        )

        # box_id = self.query_fetch_one(query=query)
        # self.conn_close()
        # return box_id

        ##################
        ##################
        ##################

        return self.query_fetch_one(query=query)

    def add_button_box_usage(self, box_name: str):

        self.logger.debug(f"table_add_button_box_usage - box_name '{box_name}'")

        if self.conn is None:
            self.conn_create()

        button_box_id = self.table_get_button_box_id(box_name=box_name)
        values = (str(datetime.now()), button_box_id)

        query = """
            INSERT INTO "{table_name}" {columns_list} VALUES {values}
        """.format(
            table_name=self.configuration.database_table_name_button_box_use,
            columns_list=("date", "box_used_id"),
            values=values
        )
        self.query_exec(query)
        self.conn_close()
